import { Chessboard, isEmpty, Square, squareAtPosition } from "./chessboard";
import { Move } from "./movements";
import { equals, left, right, top, bottom, Position, position } from "./position";

/*function isEmptyFile( chessboard: chessboard, move : move) {
    let start
}*/

/**
 * Checks whether a Black Pawn can perform a given move.
 * A pawn can move forward to the unoccupied square immediately in front of 
 * it on the same file, or on its first move it can advance two squares along 
 * the same file, provided both squares are unoccupied (black dots in the 
 * diagram); or the pawn can capture an opponent's piece on a square diagonally 
 * in front of it on an adjacent file, by moving to that square (black "x"s). 
 * 
 * 
 * @param board The chessboard of the current game
 * @param move 
 */
export function blackPawnMove(board: Chessboard, move: Move): boolean {

    if (equals(move.to!, bottom(move.from!))) {
        //console.log("Single forward");
        return isEmpty(board, move.to!);
    }

    if (move.from!.rank === 6 && equals(move.to!, bottom(bottom(move.from!)))) {
        //console.log("Double forward");
        return isEmpty(board, bottom(move.from!)) && isEmpty(board, move.to!);
    }

    if (equals(move.to!, left(bottom(move.from!))) || equals(move.to!, right(bottom(move.from!)))) {
        let destination: Square = squareAtPosition(board, move.to!);
        return !(destination.isEmpty || !destination.piece!.isWhite)
    }

    return false;
}

/**
 * A pawn can move forward to the unoccupied square immediately in front of 
 * it on the same file, or on its first move it can advance two squares along 
 * the same file, provided both squares are unoccupied (black dots in 
 * the diagram); or the pawn can capture an opponent's piece on a square diagonally 
 * in front of it on an adjacent file, by moving to that square (black "x"s). 
 * 
 * 
 * @param board The chessboard of the current game
 * @param move 
 */
export function whitePawnMove(board: Chessboard, move: Move): boolean {

    if (equals(move.to!, top(move.from!))) { 
        return isEmpty(board, move.to!);
    }
 
    if (move.from!.rank === 1 && equals(move.to!, top(top(move.from!)))) {
        return isEmpty(board, top(move.from!)) && isEmpty(board, move.to!);
    }

    if (equals(move.to!, left(top(move.from!))) || equals(move.to!, right(top(move.from!)))) {
        let destination: Square = squareAtPosition(board, move.to!);
        return !(destination.isEmpty || destination.piece!.isWhite)
    }

    return false;
}

/**
 * Checks whether a King can perform a given move.
 * The king moves one square in any direction. 
 * 
 * @param board The chessboard of the current game
 * @param move 
 */
export function blackKingMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function


    if (equals(move.to! , top(move.from!)) || equals(move.to! , bottom(move.from!)) ||            //
    equals(move.to! , left(top(move.from!))) || equals(move.to! , right(top(move.from!))) ||      // bloc qui vérifie si c'est un mouvement d'une case dans n'importe quelle direction
    equals(move.to! , left(bottom(move.from!))) || equals(move.to! , right(bottom(move.from!))) ||// 
    equals(move.to! , left(move.from!)) || equals(move.to! , right(move.from!))) {                // 
        
        if(isEmpty(board, move.to!) == false){                            //bloc pour
            let destination: Square = squareAtPosition(board, move.to!);  //manger
            return !(destination.isEmpty || !destination.piece!.isWhite)  //une pièce noire
        }
        
        return isEmpty(board, move.to!);
    }

    return false;
}

export function whiteKingMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function

    if (equals(move.to! , top(move.from!)) || equals(move.to! , bottom(move.from!)) ||             //
    equals(move.to! , left(top(move.from!))) || equals(move.to! , right(top(move.from!))) ||       //bloc qui vérifie si c'est un mouvement d'une case dans n'importe quelle direction
    equals(move.to! , left(bottom(move.from!))) || equals(move.to! , right(bottom(move.from!))) || //
    equals(move.to! , left(move.from!)) || equals(move.to! , right(move.from!))) {                 //
        
        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || destination.piece!.isWhite)  //une pièce blanche
        }
        
        return isEmpty(board, move.to!);
    }

    return false;
}

/**
 * Checks whether a Queen can perform a given move.
 * The queen combines the power of a rook and bishop and can move any 
 * number of squares along a rank, file, or diagonal, but cannot leap over other pieces.
 * 
 * @param board The chessboard of the current game
 * @param move 
 */
export function whiteQueenMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function
    let i : number 
    let caseVide : boolean

    if (Math.abs(move.from!.rank - move.to!.rank) == Math.abs(move.from!.file - move.to!.file)){ //ligne qui vérifie si c'est un mouvement en diagonal
        
         // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
        if(move.from!.rank < move.to!.rank){

        if (move.from!.file < move.to!.file){
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank+i);// pour les diagonale vers le haut à droite
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank+i);// pour les diagonale vers le haut à gauche
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }
        }else{
            if (move.from!.file < move.to!.file){
                for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                    let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank-i);// pour les diagonale vers le bas à droite
                    caseVide = isEmpty(board, caseSurChemin);
                    if (!(caseVide)){
                        return false;
                    }
                }
            }else{
                for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                    let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank-i);// pour les diagonale vers le bas à gauche
                    caseVide = isEmpty(board, caseSurChemin);
                    if (!(caseVide)){
                        return false;
                    }
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }

        return isEmpty(board, move.to!);
    }

    

    if(move.from!.rank === move.to!.rank){ //ligne qui vérifie si c'est un mouvement horizontal

         // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
        if (move.from!.file-move.to!.file < 0){
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank);// pour les mouv horizontaux vers la droite
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank);// pour les mouv horizontaux vers la gauche
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || destination.piece!.isWhite)  //une pièce blanche
        }

        return isEmpty(board, move.to!);
    }

    if(move.from!.file === move.to!.file){ //ligne qui vérifie si c'est un mouvement verticale

         // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
         if(move.from!.rank-move.to!.rank < 0){
            for(i=1; i <= Math.abs(move.from!.rank-move.to!.rank)-1; i++){
                let caseSurChemin : Position = position(move.from!.file,move.from!.rank+i);// pour les mouv verticaux vers le haut
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.rank-move.to!.rank)-1; i++){
                let caseSurChemin : Position = position(move.from!.file,move.from!.rank-i);// pour les mouv verticaux vers le bas
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc
            let destination: Square = squareAtPosition(board, move.to!); //pour manger
            return !(destination.isEmpty || destination.piece!.isWhite)  //une pièce blanche
            }

        return isEmpty(board, move.to!);
    }

    return false;
}

export function blackQueenMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function
    let i : number 
    let caseVide : boolean

    if (Math.abs(move.from!.rank - move.to!.rank) == Math.abs(move.from!.file - move.to!.file)){ //ligne qui vérifie si c'est un mouvement en diagonal
        
         // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
         if(move.from!.rank < move.to!.rank){

            if (move.from!.file < move.to!.file){
                for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                    let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank+i);// pour les diagonale vers le haut à droite
                    caseVide = isEmpty(board, caseSurChemin);
                    if (!(caseVide)){
                        return false;
                    }
                }
            }else{
                for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                    let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank+i);// pour les diagonale vers le haut à gauche
                    caseVide = isEmpty(board, caseSurChemin);
                    if (!(caseVide)){
                        return false;
                    }
                }
            }
            }else{
                if (move.from!.file < move.to!.file){
                    for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                        let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank-i);// pour les diagonale vers le bas à droite
                        caseVide = isEmpty(board, caseSurChemin);
                        if (!(caseVide)){
                            return false;
                        }
                    }
                }else{
                    for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                        let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank-i);// pour les diagonale vers le bas à gauche
                        caseVide = isEmpty(board, caseSurChemin);
                        if (!(caseVide)){
                            return false;
                        }
                    }
                }
            }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }

        return isEmpty(board, move.to!);
    }

    if(move.from!.rank === move.to!.rank){ //ligne qui vérifie si c'est un mouvement horizontal

         // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
        if (move.from!.file-move.to!.file < 0){
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank);// pour les mouv horizontaux vers la droite
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank);// pour les mouv horizontaux vers la gauche
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }

        return isEmpty(board, move.to!);
    }

    if(move.from!.file === move.to!.file){ //ligne qui vérifie si c'est un mouvement vertical

         // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
         if(move.from!.rank-move.to!.rank < 0){
            for(i=1; i <= Math.abs(move.from!.rank-move.to!.rank)-1; i++){
                let caseSurChemin : Position = position(move.from!.file,move.from!.rank+i);// pour les mouv verticaux vers le haut
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.rank-move.to!.rank)-1; i++){
                let caseSurChemin : Position = position(move.from!.file,move.from!.rank-i);// pour les mouv verticaux vers le bas
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc 
            let destination: Square = squareAtPosition(board, move.to!); //pour manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
            }

        return isEmpty(board, move.to!);
    }

    return false;
}

/**
 * Checks whether a Empress can perform a given move.
 * An Empress can move any number of squares along a rank or file, 
 * but cannot leap over other pieces.
 * An Empress can also move to any of the closest squares that are not on the 
 * same rank, file, or diagonal. (Thus the move forms an "L"-shape: 
 * two squares vertically and one square horizontally, or two 
 * squares horizontally and one square vertically.) 
 * 
 * @param board The chessboard of the current game
 * @param move 
 */
export function whiteEmpressMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function
    let i : number 
    let caseVide : boolean

    if(move.from!.rank === move.to!.rank){ //ligne qui vérifie si c'est un mouvement horizontal       

        // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
        if (move.from!.file-move.to!.file < 0){
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank);// pour les mouv horizontaux vers la droite
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank);// pour les mouv horizontaux vers la gauche
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || destination.piece!.isWhite)  //une pièce blanche
        }

        return isEmpty(board, move.to!);
    }

    if(move.from!.file === move.to!.file){ //ligne qui vérifie si c'est un mouvement vertical

         // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
         if(move.from!.rank-move.to!.rank < 0){
            for(i=1; i <= Math.abs(move.from!.rank-move.to!.rank)-1; i++){
                let caseSurChemin : Position = position(move.from!.file,move.from!.rank+i);// pour les mouv verticaux vers le haut
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.rank-move.to!.rank)-1; i++){
                let caseSurChemin : Position = position(move.from!.file,move.from!.rank-i);// pour les mouv verticaux vers le bas
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || destination.piece!.isWhite)  //une pièce
        }
        
        return isEmpty(board, move.to!);
    }
      
    if(equals(move.to!, left(top(top(move.from!))))||equals(move.to!, right(top(top(move.from!))))||          //
    equals(move.to!, top(left(left(move.from!))))||equals(move.to!, bottom(left(left(move.from!))))||         //bloc qui vérifie si c'est un mouvement cavalier en L
    equals(move.to!, top(right(right(move.from!))))||equals(move.to!, bottom(right(right(move.from!))))||     //
    equals(move.to!, left(bottom(bottom(move.from!))))||equals(move.to!, right(bottom(bottom(move.from!))))){ //

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || destination.piece!.isWhite)  //une pièce blanche
        }

        return isEmpty(board, move.to!);
    }
    return false
}

export function blackEmpressMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function
    let i : number 
    let caseVide : boolean

    if(move.from!.rank === move.to!.rank){ //ligne qui vérifie si c'est un mouvement horizontal

        // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
        if (move.from!.file-move.to!.file < 0){
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank);// pour les mouv horizontaux vers la droite
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank);// pour les mouv horizontaux vers la gauche
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }
        return isEmpty(board, move.to!);
    }

    if(move.from!.file === move.to!.file){ //ligne qui vérifie si c'est un mouvement vertical

         // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
        if(move.from!.rank-move.to!.rank < 0){
            for(i=1; i <= Math.abs(move.from!.rank-move.to!.rank)-1; i++){
                let caseSurChemin : Position = position(move.from!.file,move.from!.rank+i);// pour les mouv verticaux vers le haut
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.rank-move.to!.rank)-1; i++){
                let caseSurChemin : Position = position(move.from!.file,move.from!.rank-i);// pour les mouv verticaux vers le bas
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }
        return isEmpty(board, move.to!);
    }
      
    if(equals(move.to!, left(top(top(move.from!))))||equals(move.to!, right(top(top(move.from!))))||          //
    equals(move.to!, top(left(left(move.from!))))||equals(move.to!, bottom(left(left(move.from!))))||         //bloc qui vérifie si c'est un mouvement cavalier en L
    equals(move.to!, top(right(right(move.from!))))||equals(move.to!, bottom(right(right(move.from!))))||     //
    equals(move.to!, left(bottom(bottom(move.from!))))||equals(move.to!, right(bottom(bottom(move.from!))))){ //

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }

        return isEmpty(board, move.to!);
    }
    return false
}

/**
 * Checks whether a Princess can perform a given move.
 * A princess can move any number of squares diagonally, 
 * but cannot leap over other pieces.
 * A princess can also move to any of the closest squares that are not on the 
 * same rank, file, or diagonal. (Thus the move forms an "L"-shape: 
 * two squares vertically and one square horizontally, or two 
 * squares horizontally and one square vertically.) 
 * 
 * @param board The chessboard of the current game
 * @param move 
 */
export function blackPrincessMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function
    let i : number 
    let caseVide : boolean

    if (Math.abs(move.from!.rank - move.to!.rank) == Math.abs(move.from!.file - move.to!.file)){ //ligne qui vérifie si c'est un mouvement en diagonal
        
      // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
      if(move.from!.rank < move.to!.rank){

        if (move.from!.file < move.to!.file){
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank+i);// pour les diagonale vers le haut à droite
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }else{
            for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank+i);// pour les diagonale vers le haut à gauche
                caseVide = isEmpty(board, caseSurChemin);
                if (!(caseVide)){
                    return false;
                }
            }
        }
        }else{
            if (move.from!.file < move.to!.file){
                for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                    let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank-i);// pour les diagonale vers le bas à droite
                    caseVide = isEmpty(board, caseSurChemin);
                    if (!(caseVide)){
                        return false;
                    }
                }
            }else{
                for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                    let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank-i);// pour les diagonale vers le bas à gauche
                    caseVide = isEmpty(board, caseSurChemin);
                    if (!(caseVide)){
                        return false;
                    }
                }
            }
        }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }

        return isEmpty(board, move.to!);
    }

    if(equals(move.to!, left(top(top(move.from!))))||equals(move.to!, right(top(top(move.from!))))||          //
    equals(move.to!, top(left(left(move.from!))))||equals(move.to!, bottom(left(left(move.from!))))||         //bloc qui vérifie si c'est un mouvement cavalier en L
    equals(move.to!, top(right(right(move.from!))))||equals(move.to!, bottom(right(right(move.from!))))||     //
    equals(move.to!, left(bottom(bottom(move.from!))))||equals(move.to!, right(bottom(bottom(move.from!))))){ //

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }

        return isEmpty(board, move.to!);
    }

    return false
}

export function whitePrincessMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function
    let i : number 
    let caseVide : boolean

    if (Math.abs(move.from!.rank - move.to!.rank) == Math.abs(move.from!.file - move.to!.file)){ //ligne qui vérifie si c'est un mouvement en diagonal
    
         // bloc qui vérifie s'il n'y a pas de pièce sur le chemin 
         if(move.from!.rank < move.to!.rank){

            if (move.from!.file < move.to!.file){
                for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                    let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank+i);// pour les diagonale vers le haut à droite
                    caseVide = isEmpty(board, caseSurChemin);
                    if (!(caseVide)){
                        return false;
                    }
                }
            }else{
                for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                    let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank+i);// pour les diagonale vers le haut à gauche
                    caseVide = isEmpty(board, caseSurChemin);
                    if (!(caseVide)){
                        return false;
                    }
                }
            }
            }else{
                if (move.from!.file < move.to!.file){
                    for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                        let caseSurChemin : Position = position(move.from!.file+i,move.from!.rank-i);// pour les diagonale vers le bas à droite
                        caseVide = isEmpty(board, caseSurChemin);
                        if (!(caseVide)){
                            return false;
                        }
                    }
                }else{
                    for(i=1; i <= Math.abs(move.from!.file-move.to!.file)-1; i++){
                        let caseSurChemin : Position = position(move.from!.file-i,move.from!.rank-i);// pour les diagonale vers le bas à gauche
                        caseVide = isEmpty(board, caseSurChemin);
                        if (!(caseVide)){
                            return false;
                        }
                    }
                }
            }

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }

        return isEmpty(board, move.to!);
    }

    if(equals(move.to!, left(top(top(move.from!))))||equals(move.to!, right(top(top(move.from!))))||          //
    equals(move.to!, top(left(left(move.from!))))||equals(move.to!, bottom(left(left(move.from!))))||         //bloc qui vérifie si c'est un mouvement cavalier en L
    equals(move.to!, top(right(right(move.from!))))||equals(move.to!, bottom(right(right(move.from!))))||     //
    equals(move.to!, left(bottom(bottom(move.from!))))||equals(move.to!, right(bottom(bottom(move.from!))))){ //

        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || destination.piece!.isWhite)  //une pièce blanche
        }

        return isEmpty(board, move.to!);
    }
    return false
}

/**
 * Checks whether a Camel can perform a given move.
 * The Camel move forms an "L"-shape: 
 * three squares vertically and one square horizontally, or three 
 * squares horizontally and one square vertically.) 
 * 
 * The camel can leap over other pieces.
 * 
 * @param board The chessboard of the current game
 * @param move 
 */
export function whiteCamelMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function


    if(equals(move.to!, left(top(top(top(move.from!)))))||equals(move.to!, right(top(top(top(move.from!)))))||                //
    equals(move.to!, top(left(left(left(move.from!)))))||equals(move.to!, bottom(left(left(left(move.from!)))))||             //bloc qui vérifie si c'est un mouvement chameau en L
    equals(move.to!, top(right(right(right(move.from!)))))||equals(move.to!, bottom(right(right(right(move.from!)))))||       //
    equals(move.to!, left(bottom(bottom(bottom(move.from!)))))||equals(move.to!, right(bottom(bottom(bottom(move.from!)))))){ //
        
        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || destination.piece!.isWhite)  //une pièce blanche
        }
        
        return isEmpty(board, move.to!);
    }

    return false;
} 

export function blackCamelMove(board: Chessboard, move: Move): boolean {
    // #TODO: Implement this function


    if(equals(move.to!, left(top(top(top(move.from!)))))||equals(move.to!, right(top(top(top(move.from!)))))||                //
    equals(move.to!, top(left(left(left(move.from!)))))||equals(move.to!, bottom(left(left(left(move.from!)))))||             //bloc qui vérifie si c'est un mouvement chameau en L
    equals(move.to!, top(right(right(right(move.from!)))))||equals(move.to!, bottom(right(right(right(move.from!)))))||       //
    equals(move.to!, left(bottom(bottom(bottom(move.from!)))))||equals(move.to!, right(bottom(bottom(bottom(move.from!)))))){ //
        
        if(isEmpty(board, move.to!) == false){                           //bloc pour
            let destination: Square = squareAtPosition(board, move.to!); //manger
            return !(destination.isEmpty || !destination.piece!.isWhite) //une pièce noire
        }
        
        return isEmpty(board, move.to!);
    }

    return false;
} 