import { Expect, Test, Setup} from "alsatian";
import { Chessboard, createEmptyChessboard, putPiece } from '../../main/ts/chessboard';
import * as pieces from '../../main/ts/piece'
import { Position, position } from '../../main/ts/position';
import { Move, move } from '../../main/ts/movements';
import * as isPossible from "/home/utilisateur/projet-2019/src/main/ts/move-validation"

let chessboard : Chessboard;

const positionA4 : Position = position(0, 3) // A4
const positionA5 : Position = position(0, 4) // A5
const positionA6 : Position = position(0, 5) // A6
const positionA7 : Position = position(0, 6) // A7
const positionA8 : Position = position(0, 7) // A8

const positionB1 : Position = position(1, 0) // B1
const positionB2 : Position = position(1, 1) // B2
const positionB3 : Position = position(1, 2) // B3
const positionB4 : Position = position(1, 3) // B4
const positionB5 : Position = position(1, 4) // B5
const positionB6 : Position = position(1, 5) // B6

const positionC3 : Position = position(2, 2) // C3
const positionC4 : Position = position(2, 3) // C4
const positionC5 : Position = position(2, 4) // C5
const positionC6 : Position = position(2, 5) // C6
const positionC7 : Position = position(2, 6) // C7

const positionD1 : Position = position(3, 0) // D1
const positionD2 : Position = position(3, 1) // D2
const positionD3 : Position = position(3, 2) // D3
const positionD4 : Position = position(3, 3) // D4
const positionD5 : Position = position(3, 4) // D5
const positionD6 : Position = position(3, 5) // D6
const positionD7 : Position = position(3, 6) // D7


const positionE1 : Position = position(4, 0) // E1
const positionE3 : Position = position(4, 2) // E3
const positionE4 : Position = position(4, 3) // E4
const positionE7 : Position = position(4, 6) // E7
const positionE8 : Position = position(4, 7) // E8

const positionF1 : Position = position(5, 0) // F1
const positionF4 : Position = position(5, 3) // F4
const positionF2 : Position = position(5, 1) // F2
const positionF6 : Position = position(5, 5) // F6
const positionF7 : Position = position(5, 6) // F7
const positionG3 : Position = position(6, 2) // G3
const positionG5 : Position = position(6, 4) // G5

const positionH1 : Position = position(7, 0) // H1
const positionH3 : Position = position(7, 2) // H3
const positionH4 : Position = position(7, 3) // H4
const positionH5 : Position = position(7, 4) // H5
const positionH7 : Position = position(7, 6) // H7

// Horizontal moves
const moveE4_H4 : Move = move(positionE4, positionH4);
const moveE4_A4 : Move = move(positionE4, positionA4);

// Vertical moves
const moveE4_E1 : Move = move(positionE4, positionE1);
const moveE4_E8 : Move = move(positionE4, positionE8);

// ThreeHorizontalAndOneVertical
const moveE4_H3 : Move = move(positionE4, positionH3);
const moveE4_H5 : Move = move(positionE4, positionH5);
const moveE4_B3 : Move = move(positionE4, positionB3);
const moveE4_B5 : Move = move(positionE4, positionB5);

// ThreeVerticalAndOneHorizontal
const moveE4_F1 : Move = move(positionE4, positionF1);
const moveE4_F7 : Move = move(positionE4, positionF7);
const moveE4_D1 : Move = move(positionE4, positionD1);
const moveE4_D7 : Move = move(positionE4, positionD7);

// Diagonally
const moveE4_A8 : Move = move(positionE4, positionA8);
const moveE4_B1 : Move = move(positionE4, positionB1);
const moveE4_H7 : Move = move(positionE4, positionH7);
const moveE4_H1 : Move = move(positionE4, positionH1);

export class TestCamelMoves {
    @Setup
    beforeEach() {
        // TODO:
        // Initialize an empty chessboard
        // Place a white Camel on E4
        chessboard=createEmptyChessboard();
        putPiece(chessboard, positionE4, pieces.whiteCamel);
    }

    @Test("A Camel can move three squares horizontally and one square vertically")
    testCanMoveThreeHorizontalAndOneVertical() {
        // TODO:
        // Check the following moves are possible:
        // - moveE4_H3
        // - moveE4_H5
        // - moveE4_B3
        // - moveE4_B5
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_H3)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_H5)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_B3)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_B5)).toBeTruthy();
    }

    @Test("A Camel can move three squares vertically and one square horizontally")
    testCanMoveThreeVerticalAndOneHorizontal() {
        // TODO:
        // Check the following moves are possible:
        // - moveE4_F1
        // - moveE4_F7
        // - moveE4_D1
        // - moveE4_D7
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_F1)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_F7)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_D1)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_D7)).toBeTruthy();
    }

    @Test("A Camel can leap other pieces")
    testCanLeapOtherPieces() {
        // TODO:
        // Place a black Pawn on E7, H4, E1 and B4
        // Check the following moves are possible:
        // - moveE4_F1
        // - moveE4_F7
        // - moveE4_D1
        // - moveE4_D7
        // - moveE4_H3
        // - moveE4_H5
        // - moveE4_B3
        // - moveE4_B5
        putPiece(chessboard, positionE7, pieces.blackPawn); 
        putPiece(chessboard, positionH4, pieces.blackPawn); 
        putPiece(chessboard, positionE1, pieces.blackPawn); 
        putPiece(chessboard, positionB4, pieces.blackPawn); 
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_F1)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_F7)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_D1)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_D7)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_H3)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_H5)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_B3)).toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_B5)).toBeTruthy();
    }

    @Test("A Camel cannot move diagonally")
    testCannotMoveDiagonally() {
        // TODO:
        // Check the following moves are impossible: 
        // moveE4_A8, moveE4_B1, moveE4_H7, moveE4_H1
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_A8)).not.toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_B1)).not.toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_H7)).not.toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_H1)).not.toBeTruthy();
    }

    @Test("A Camel cannot move horizontally")
    testCannotMoveHorizontally() {
        // TODO:
        // Check the following moves are impossible: moveE4_H4, moveE4_A4 
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_H4)).not.toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_A4)).not.toBeTruthy();
    }

    @Test("A Camel cannot move vertically")
    testCannotMoveVertically() {
        // TODO:
        // Check the following moves are impossible: moveE4_E1, moveE4_E8     
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_E8)).not.toBeTruthy();
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_E1)).not.toBeTruthy(); 
    }

    @Test("A Camel can capture a piece from another color")
    testCanCaptureAnotherColor() {
        // TODO:
        // Place a black Pawn on D7 and H5
        putPiece(chessboard, positionD7, pieces.blackPawn); 
        putPiece(chessboard, positionH5, pieces.blackPawn); 
        // Check the move moveE4_A8 is impossible      
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_D7)).toBeTruthy(); 
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_H5)).toBeTruthy(); 
    }

    @Test("A Camel cannot capture a piece from the same color")
    testCannotCaptureSameColor() {
        // TODO:
        // Place a white Pawn on D7 and H5
        putPiece(chessboard, positionD7, pieces.whitePawn); 
        putPiece(chessboard, positionH5, pieces.whitePawn); 
        // Check the move moveE4_A8 is impossible      
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_D7)).not.toBeTruthy(); 
        Expect(isPossible.whiteCamelMove(chessboard, moveE4_H5)).not.toBeTruthy(); 
    }
}
